@@ -7,6 +7,9 @@
 /* 
  * Revisions:
  *
+ * 08/19/00  [rg]
+ *         - Added vlogit.
+ *
  * 01/25/00  [rg]
  *        - Added parameters for file name and line number.
  *
@@ -32,15 +35,23 @@
 #include <stdio.h>
 #include <sys/types.h>
 
-#  include <stdlib.h>
-#  include <stdarg.h>
+#include <stdlib.h>
+#include <stdarg.h>
 
 #include "popper.h"
 
 
 int logit ( FILE        *str,       /* STREAM to write to, or NULL for syslog */
-            log_level    stat,      /* log level */
+            log_level    loglev,    /* log level */
             const char  *fn,        /* file name whence called */
             size_t       ln,        /* line number whence called */
             const char  *format,    /* format string to log */
             ... );                  /* parameters for format string */
+
+int vlogit ( FILE        *str,       /* STREAM to write to, or NULL for syslog */
+             log_level    loglev,    /* log level */
+             const char  *fn,        /* file name whence called */
+             size_t       ln,        /* line number whence called */
+             const char  *format,    /* format string to log */
+             va_list      ap );  
+
