@@ -24,7 +24,7 @@ typedef struct tagCHMangle {
     void            *m_pvOut;
     TextCharSetType m_partCharSet;
     TextCharSetType m_reqCharSet;
-}CHMangle;
+} CHMangle;
 
 static CHMangle hm;
 
@@ -35,6 +35,7 @@ void *CharManglerInit ( OutputFn oFn, void *oFnState, TextCharSetType partCharSe
     hm.m_pvOut = oFnState;
     hm.m_partCharSet = partCharSet;
     hm.m_reqCharSet = reqCharSet;
+    return &hm;
 }
 
 #define BLAT '_'
@@ -141,7 +142,6 @@ static const unsigned int CP1252[] = {
 
 static unsigned int Iso8859_to_Unicode( TextCharSetType CharSet, unsigned char character )
 {
-    unsigned int nRet;
     switch( CharSet ) {
     case us_ascii:
     return (character & 0x7F);
@@ -202,146 +202,184 @@ static void Unicode_to_Utf8( unsigned int unicode, unsigned char *buf, int *nSiz
     buf[i] = 0x80 | ( (unicode >> (0x06 * (*nSize - i - 1))) & 0x3F );
 }
 
-void CharMangler(void *state, char *szText, long len)
+void CharMangler ( void *state, char *szText, long len )
 {
     int i;
-    switch(hm.m_reqCharSet) {
-    case us_ascii:
-    switch(hm.m_partCharSet) {
-    case utf_8:
-        for( i = 0; i < len; i++ ) {
-        switch(szText[i] & 0xC0) {
-        case 0xC0 :
-            szText[i] = BLAT;
+    (void) len;
+    (void) state;
+
+
+    switch ( hm.m_reqCharSet ) {
+
+        case us_ascii:
+            switch ( hm.m_partCharSet ) {
+                case utf_8:
+                    for ( i = 0; i < len; i++ ) {
+                        switch ( szText[i] & 0xC0 ) {
+                            case 0xC0:
+                                szText[i] = BLAT;
+                                break;
+                            case 0x80:
+                                break;
+                            default:
+                                hm.m_pfOut ( hm.m_pvOut, &szText[i], 1 );
+                                break;
+                        } /* switch ( szText[i] & 0xC0 ) */
+                    } /* for loop */
+                    break;
+                case us_ascii: /* Pass */
+                    hm.m_pfOut ( hm.m_pvOut, szText, len );
+                    break;  
+                default:
+                    /* Filter the octets with b7 set */
+                    for ( i = 0; i < len; i++ ) {
+                        if ( ( (unsigned char) szText[i] != 0xA0 ) && 
+                             ( (unsigned char) szText[i]  & 0x80 ) ) {
+                            szText [ i ] = BLAT;
+                        }
+                    } /* for loop */
+                    hm.m_pfOut ( hm.m_pvOut, szText, len );
+            } /* switch ( hm.m_partCharSet ) */
             break;
-        case 0x80 :
+
+        case iso_8859_1:
+        case iso_8859_2:
+        case iso_8859_3:
+        case iso_8859_4:
+        case iso_8859_9:
+        case iso_8859_10:  /* Latin 1 thru 5 */
+            switch ( hm.m_partCharSet ) {
+                case us_ascii:
+                case iso_8859_1:
+                case iso_8859_2:
+                case iso_8859_3:
+                case iso_8859_4:
+                case iso_8859_9:
+                case iso_8859_10: /* Pass */
+                    hm.m_pfOut ( hm.m_pvOut, szText, len );
+                    break;
+                case utf_8:
+                    /* todo */
+                    break;
+                case iso_8859_5:
+                case iso_8859_6:
+                case iso_8859_7:
+                case iso_8859_8: /* Octets with b7 set replaced with 'blat' */
+                default:
+                    for ( i = 0; i < len; i++ ) {
+                        if ( szText[i] & 0x80 ) {
+                            szText[i] = '_';
+                        }
+                    } /* for loop */
+                    hm.m_pfOut ( hm.m_pvOut, szText, len );
+                    break;
+            } /* switch ( hm.m_partCharSet ) */
             break;
+
+        case iso_8859_5:
+        case iso_8859_6:
+        case iso_8859_7:
+        case iso_8859_8:
+            switch ( hm.m_partCharSet ) {
+                case utf_8:
+                    break;
+                default:
+                    /* Filter the octets with b7 set */
+                    for ( i = 0; i < len; i++ ) {
+                        if ( szText[i] & 0x80 ) {
+                            szText[i] = '_';
+                        }
+                    } /* for loop */
+                    hm.m_pfOut ( hm.m_pvOut, szText, len );
+            } /* switch ( hm.m_partCharSet ) */
+            break;
+
+        case cp1252:
+            switch ( hm.m_partCharSet ) {
+                case us_ascii:
+                case iso_8859_1:
+                case iso_8859_2:
+                case iso_8859_3:
+                case iso_8859_4:
+                case iso_8859_9:
+                case iso_8859_10:
+                    hm.m_pfOut ( hm.m_pvOut, szText, len );
+                    break;
+                default:
+                    /* Filter the octets with b7 set */
+                    for ( i = 0; i < len; i++ ) {
+                        if ( szText[i] & 0x80 ) {
+                            szText[i] = '_';
+                        }
+                    } /* for loop */
+                    hm.m_pfOut ( hm.m_pvOut, szText, len) ;
+            } /* switch ( hm.m_partCharSet ) */
+            break;
+
+        case iso_2022_jp:
+            break;
+
+        case utf_8:
+            switch ( hm.m_partCharSet ) {
+                case utf_8:
+                case us_ascii:
+                    /* Pass */
+                    hm.m_pfOut ( hm.m_pvOut, szText, len );
+                    break;
+                case iso_8859_1:
+                case iso_8859_2:
+                case iso_8859_3:
+                case iso_8859_4:
+                case iso_8859_5:
+                case iso_8859_6:
+                case iso_8859_7:
+                case iso_8859_8:
+                case iso_8859_9:
+                    { /* local block */
+                    unsigned char szBuf[6];
+                    int  nS = 6;
+                    unsigned int unicode;
+
+                    for ( i = 0; i < len; i++ ) {
+                        if ( ( (unsigned char) szText[i] ) > 0xA0 ) {
+                            unicode = Iso8859_to_Unicode ( hm.m_partCharSet, szText[i] );
+                            Unicode_to_Utf8 ( unicode, szBuf, &nS );
+                            hm.m_pfOut ( hm.m_pvOut, (char *)szBuf, nS );
+                        }
+                        else
+                            hm.m_pfOut ( hm.m_pvOut, &szText[i], 1 );
+                    } /* for loop */
+                    } /* local block */
+                    break;
+                case iso_8859_10:
+                    /* to do */
+                    break;
+                case iso_8859_11:
+                    /* to do */
+                    break;
+                case iso_8859_12:
+                    /* to do */
+                    break;
+                case iso_8859_13:
+                    /* to do */
+                    break;
+                case iso_8859_14:
+                    /* to do */
+                    break;
+                case iso_8859_15:
+                    /* to do */
+                    break;
+                case cp1252:
+                    /* to do */
+                    break;
+                case iso_2022_jp:
+                    /* to do */
+                    break;
+            } /* switch ( hm.m_partCharSet ) */
+            break;
+
         default:
-            hm.m_pfOut( hm.m_pvOut, &szText[i], 1 );
             break;
-        }
-        }
-        break;
-    case us_ascii: /* Pass */
-        hm.m_pfOut(hm.m_pvOut, szText, len);
-        break;  
-    default:
-        /* Filter the octets with b7 set */
-        for(i = 0; i < len; i++) {
-        if ( ( (unsigned char) szText[i] != 0xA0 ) && 
-                     ( (unsigned char) szText[i]  & 0x80 ) ) {
-            szText[i] = BLAT;
-        }
-        }
-        hm.m_pfOut(hm.m_pvOut, szText, len);
-    }
-    break;
-    case iso_8859_1:
-    case iso_8859_2:
-    case iso_8859_3:
-    case iso_8859_4:
-    case iso_8859_9:
-    case iso_8859_10:  /* Latin 1 thru 5 */
-    switch(hm.m_partCharSet) {
-    case us_ascii:
-    case iso_8859_1:
-    case iso_8859_2:
-    case iso_8859_3:
-    case iso_8859_4:
-    case iso_8859_9:
-    case iso_8859_10: /* Pass */
-        hm.m_pfOut(hm.m_pvOut, szText, len);
-        break;
-    case utf_8:
-        /* todo */
-        break;
-    case iso_8859_5:
-    case iso_8859_6:
-    case iso_8859_7:
-    case iso_8859_8: /* Octets with b7 set replaced with 'blat' */
-    default:
-        for(i = 0; i < len; i++) {
-        if ( szText[i] & 0x80 ) {
-            szText[i] = '_';
-        }
-        }
-        hm.m_pfOut(hm.m_pvOut, szText, len);
-        break;
-    }
-    break;
-    case iso_8859_5:
-    case iso_8859_6:
-    case iso_8859_7:
-    case iso_8859_8:
-    switch(hm.m_partCharSet) {
-    case utf_8:
-        break;
-    default:
-        /* Filter the octets with b7 set */
-        for(i = 0; i < len; i++) {
-        if ( szText[i] & 0x80 ) {
-            szText[i] = '_';
-        }
-        }
-        hm.m_pfOut(hm.m_pvOut, szText, len);
-    }
-    break;
-    case cp1252:
-    switch(hm.m_partCharSet) {
-    case us_ascii:
-    case iso_8859_1:
-    case iso_8859_2:
-    case iso_8859_3:
-    case iso_8859_4:
-    case iso_8859_9:
-    case iso_8859_10:
-        hm.m_pfOut(hm.m_pvOut, szText, len);
-        break;
-    default:
-        /* Filter the octets with b7 set */
-        for(i = 0; i < len; i++) {
-        if ( szText[i] & 0x80 ) {
-            szText[i] = '_';
-        }
-        }
-        hm.m_pfOut(hm.m_pvOut, szText, len);
-    }
-    break;
-    case iso_2022_jp:
-    break;
-    case utf_8:
-    switch( hm.m_partCharSet) {
-    case us_ascii:
-        /* Pass */
-        hm.m_pfOut(hm.m_pvOut, szText, len);
-        break;
-    case iso_8859_1:
-    case iso_8859_2:
-    case iso_8859_3:
-    case iso_8859_4:
-    case iso_8859_5:
-    case iso_8859_6:
-    case iso_8859_7:
-    case iso_8859_8:
-    case iso_8859_9:
-        {
-        unsigned char szBuf[6];
-        int  nS = 6;
-        unsigned int unicode;
-        for ( i = 0; i < len; i++ ) {
-            if ( ((unsigned char)szText[i]) > 0xA0 ) {
-                unicode = Iso8859_to_Unicode ( hm.m_partCharSet, szText[i] );
-                Unicode_to_Utf8 ( unicode, szBuf, &nS );
-                hm.m_pfOut ( hm.m_pvOut, (char *)szBuf, nS );
-            }
-            else hm.m_pfOut( hm.m_pvOut, &szText[i], 1 );
-        }
-        }
-        break;
-    }
-    break;
-    default:
-    break;
-    }
+    } /* switch ( hm.m_reqCharSet ) */
 }
 
