@@ -16,6 +16,11 @@
      to plain text.
 
      Revisions:
+            Jun 20, 2000.  [rg]
+            - Asking for an unsupported transormation (such as text/enriched)
+              gets you text/plain.
+            Jun 10, 2000.  [rg]
+            - FillMangleInfo now returns 0 on success instead of random.
             Apr 28, 2000.  [rg]
             - Ignore external body parts.
             Feb 03, 2000.  [rg]
@@ -23,7 +28,7 @@
               happier.
             May 01, 1998.  [py]
             - Handling for M/Alternate and M/Report
-            - Handling of headers for Message/* parts.
+            - Handling of headers for Message/ parts.
             Mar 20, 1998.  [py]
             - Added support for Mangle Extension.
             - Modified headerHandler to display required Mime Headers.
@@ -36,11 +41,13 @@
 #include "striphtml.h"
 #include "mangle.h"
 #include "charmangle.h"
+#include "utils.h"
 
 #ifdef PILOT
 #  include <Pilot.h>
 #else /* UNIX */
 #  include <stdio.h>
+#  include <stdlib.h>
 #  include <string.h>
 #  define StrCompare strcmp
 #  define StrCopy    strcpy
@@ -73,8 +80,8 @@ static char *charsetAttributes[] = { "us-ascii",   "iso-8859-1", "iso-8859-2",
    Note that at this time the Mime parser decodes 2047 formatting of headers
    for us into ISO-8859-1 text. 
 
-   BUG: we don't know when we're called for nested message/* entities
-   and thus can't omit whole message/* entities! This is probably a bug
+   BUG: we don't know when we're called for nested message/ entities
+   and thus can't omit whole message/ entities! This is probably a bug
    in the core MIME parser and needs to be rethought.
    ---- */
 void headerHandler( state, header , len, isContinuation )
@@ -83,12 +90,12 @@ char         *header;
 long          len;
 unsigned char isContinuation;
 {
-  char          **goodHeader, *h;
+  char       *h;
   ManglerStatePtr mS = (ManglerStatePtr)state;
   static unsigned char cEntry = 0;   /* The first one doesn't undergo transformation
                                      * but the rest would go because they are part of 
                                      * message body.*/
-  int i;
+  size_t i;
 
   /* Header separator is not supposed to have anything but a line end, but..*/
   for(h = header; h < header + len; h++) {
@@ -108,6 +115,8 @@ unsigned char isContinuation;
           case TextEnriched:
               mS->outFn(mS->outFnState, "Content-Type: text/enriched;",28);
               break;
+          default:
+              break;
           }
           mS->outFn(mS->outFnState, "charset=\"", 9);
           mS->outFn(mS->outFnState, charsetAttributes[mS->rqInfo.m_char_set], 
@@ -178,7 +187,7 @@ void MangleMapper( ourState, mError, mType, multAttribs, outFnConstructor, outFn
     char           *typeInfo    = NULL;
     char           *typeMatch   = NULL;
     char           *types       = NULL;
-    TextCharSetType charset;
+    TextCharSetType charset     = (TextCharSetType) 0;
     int             commaCount;
     char            tempBuf[100];
     ManglerStatePtr mS = (ManglerStatePtr)ourState;
@@ -242,7 +251,6 @@ void MangleMapper( ourState, mError, mType, multAttribs, outFnConstructor, outFn
             } /* majorTypeText */
             else if ( multAttribs->attribs[multAttribs->nestLevel+1].mult == multRelated ) {
                 if ( mType->paramIndex[paramType] != kParamIndexUnused ) {
-                    char c;
                     typeMatch = NULL;
                     types = mType->param[mType->paramIndex[paramType]].value;
                     /* See if it is a text part control */
@@ -347,6 +355,8 @@ void MangleMapper( ourState, mError, mType, multAttribs, outFnConstructor, outFn
                 case multRelated:
                     mS->omitMap[multAttribs->nestLevel+1] = 0x0000;
                     break;
+                default:
+                    break;
             } /* switch multAttribs->attribs[multAttribs->nestLevel+1].mult */
             return;
             /* Never output anything for multiparts, unless we want to 
@@ -375,6 +385,15 @@ void MangleMapper( ourState, mError, mType, multAttribs, outFnConstructor, outFn
 
             /* for text body parts, we map based on the requested text type */
             switch ( mS->rqInfo.m_text ) {
+                case TextEnriched:
+                    /*
+                     * We don't handle conversion to text/enriched, so
+                     * if that's what you ask for, you'll get text/plain.
+                     */
+                default:
+                    /*
+                     * If you ask for something weird, you'll get text/plain.
+                     */
                 case TextPlain:
                     *outFnConstructor = *outFnDestructor = NULL;
                     if ( !StrCompare(mType->minorType, "enriched") ) {
@@ -405,7 +424,8 @@ void MangleMapper( ourState, mError, mType, multAttribs, outFnConstructor, outFn
                         *outFnConstructor = textEnrichedToHTMLHead;
                         *outFn = textEnrichedToHTML;
                         *outFnDestructor = textEnrichedToHTMLTail;
-                        *outFnState = textEnrichedInit(mS->outFn, mS->outFnState, 1, 1);
+                        *outFnState = textEnrichedInit(mS->outFn, mS->outFnState, 
+                                                       iso_8859_1, iso_8859_1);
                     }
                     else {
                         if ( !StrCompare(mType->minorType, "plain") 
@@ -417,7 +437,8 @@ void MangleMapper( ourState, mError, mType, multAttribs, outFnConstructor, outFn
                                 *outFnDestructor  = textFlowedToHTMLTail;
                                 *outFn            = textFlowedToHTML;
                             } else {
-                                *outFnState       = textHTMLInit ( mS->outFn, mS->outFnState, 1, 1 );
+                                *outFnState       = textHTMLInit ( mS->outFn, mS->outFnState,
+                                                                   iso_8859_1, iso_8859_1 );
                                 *outFnConstructor = textPlainToHTMLHead;
                                 *outFnDestructor  = textPlainToHTMLTail;
                                 *outFn            = textPlainToHTML;
@@ -584,7 +605,8 @@ splat:
         void          *out_state;
         switch ( mS->rqInfo.m_text ) {
             case TextHTML:
-                out_state  = textHTMLInit ( mS->outFn, mS->outFnState, 1, 1 );
+                out_state  = textHTMLInit ( mS->outFn, mS->outFnState, 
+                                            iso_8859_1, iso_8859_1 );
                 out_FnCtor = textPlainToHTMLHead;
                 out_FnDtor = textPlainToHTMLTail;
                 out_Fn     = textPlainToHTML;
@@ -608,7 +630,7 @@ splat:
  */
 void FreeMangleInfo ( MangleInfoType *infoStruct )
 {
-    int i;
+    size_t i;
     if(!infoStruct->m_headers) return;
     for(i = 0; i < infoStruct->m_headers->size; i++)
         free(infoStruct->m_headers->elem[i]);
@@ -635,7 +657,7 @@ int FillMangleInfo ( char *commandLine, char bNoContent,
 
     int nLen;
     char *p;
-    int i;
+    size_t i;
 
     /* Fill the macros from MDEF Array */
 
@@ -674,11 +696,12 @@ int FillMangleInfo ( char *commandLine, char bNoContent,
                     break;
                 }
             }
-            if (i == n) return -1;
+            if (i == n) 
+                return -1;
         }
     }
     if((p = strstr(commandLine,"charset=")) != NULL) {
-        int n = sizeof(charsetAttributes)/sizeof(charsetAttributes[0]);
+        size_t n = sizeof(charsetAttributes)/sizeof(charsetAttributes[0]);
         char *q;
         p += 8;
         if((q=strchr(p,';')) != NULL || (q=strchr(p,')')) != NULL) {
@@ -698,7 +721,7 @@ int FillMangleInfo ( char *commandLine, char bNoContent,
         infoStruct->m_headers
             = (StringVectorType *)malloc(sizeof(StringVectorType));
         if(infoStruct->m_headers) {
-            char *q,c, *StrNDup(char *, const int);
+            char *q, *StrNDup(char *, const int);
             int i;
             p += 8;
             for(i=1,q = p; (*q && *q != ';' && *q != ')'); q++)
@@ -717,7 +740,7 @@ int FillMangleInfo ( char *commandLine, char bNoContent,
             }
         }
     }
-    return;
+    return 0;
 }
 
 
