@@ -6,6 +6,19 @@
 
 
 /* Changes:
+ *
+ *   07/07/00  [rcg]
+ *            - Fixed incorrect calculation of visible_msg_num.
+ *
+ *   06/30/00  [rcg]
+ *            - Replaced strncpy/strncat with strlcpy/strlcat, which
+ *             guarantee to null-terminate the buffer, and are easier
+ *             to use safely.
+ *
+ *   06/16/00  [rcg]
+ *            - Properly clean up during errors.
+ *            - Using passed-in passwd structure instead of calling
+ *              getpwnam yet again.
  *
  *   03/08/00  [rcg]
  *            - Use pop_log instead of pop_msg, since our caller does not
@@ -32,17 +45,38 @@
  */
 
 
+#include "config.h"
+
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <errno.h>
-#include <string.h>
+#include <flock.h>
 #include <pwd.h>
 #include <grp.h>
+#include <ctype.h>
+#include <string.h>
+#include <time.h>
 
-#include "config.h"
+#if HAVE_STRINGS_H
+#  include <strings.h>
+#endif
 
 #ifdef BULLDB
 #  include <sys/file.h>
+
+#  ifdef GDBM
+#    include <gdbm.h>
+#  else
+#    if HAVE_NDBM_H
+#      include <ndbm.h>
+#    endif /* HAVE_NDBM_H */
+#  endif /* GDBM */
+
+#  ifndef dbm_dirfno
+     extern int dbm_dirfno();
+#  endif
 #endif /* BULLDB */
 
 #ifdef HAVE_DIRENT_H
@@ -65,15 +99,21 @@
 #  endif /* HAVE_NDIR_H */
 #endif /* else of ifdef HAVE_DIRENT_H */
 
+#ifdef HAVE_SYS_UNISTD_H
+#  include <sys/unistd.h>
+#endif /* HAVE_SYS_UNISTD_H */
+
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+
 #ifndef HAVE_BCOPY
 #  define bcopy(src,dest,len)   (void) (memcpy(dest,src,len))
 #endif /* ndef HAVE_BCOPY */
 
-#include <ctype.h>
-
 #include "popper.h"
+#include "string_util.h"
 
-#include <flock.h>
 
 static int sequence = 0;
 static long timestamp;
@@ -94,6 +134,9 @@ struct _file_list {
 };
 
 
+int CopyOneBull ( POP *p, long bnum, char *name );
+
+
 /*
  * Based on patch by Mikolaj Rydzewski.
  *
@@ -107,18 +150,16 @@ struct _file_list {
  * getgrnam()) then the bulletin is delivered to every user.
  */
 int 
-IsBulletinForMe ( POP *p, const char *bull_name )
+IsBulletinForMe ( POP *p, struct passwd *pwd, const char *bull_name )
 
 {
     char           *token;
     char            group_name [30];
     char           *ptr;
     char          **u_list;
-    int             group_len = 0;
+    size_t          group_len = 0;
     struct group   *grp;
     struct group   *pwgrp;
-    struct passwd  *pwd;
-    static gid_t    my_group = 0;
     gid_t           b_group;
 
 
@@ -143,34 +184,19 @@ IsBulletinForMe ( POP *p, const char *bull_name )
     }
     *ptr = 0;
 
+    pwgrp = getgrgid ( pwd->pw_gid );
     /*
-     * Save off my primary group
+     * If user is not a member of a group, no bulletins are shown.
      */
-    if ( my_group == 0 ) {
-        pwd = getpwnam ( p->user );
-        /* 
-         * something got strange, but let's assume it's Ok 
-         */
-        if ( pwd == NULL ) {
-            DEBUG_LOG3 ( p, "getpwnam failed for user %.128s: %s (%d)",
-                        p->user, STRERROR(errno), errno );
-            return 1;
-        }
-
-        pwgrp = getgrgid ( pwd->pw_gid );
-        /*
-         * If user is not a member of a group, no bulletins are shown.
-         */
-        if ( pwgrp == NULL ) {
-            DEBUG_LOG3 ( p, "getgrgid failed for gid %d: %s (%d)",
-                        pwd->pw_gid, STRERROR(errno), errno );
-            return 0;
-        }
-
-        my_group = pwd->pw_gid;
-        DEBUG_LOG2 ( p, "Primary gid for user %s is %d", p->user, my_group );
+    if ( pwgrp == NULL ) {
+        DEBUG_LOG3 ( p, "getgrgid failed for gid %u: %s (%d)",
+                    (unsigned) pwd->pw_gid, STRERROR(errno), errno );
+        return 0;
     }
 
+    DEBUG_LOG2 ( p, "Primary gid for user %s is %u", 
+                 p->user, (unsigned) pwd->pw_gid );
+
     /* 
      * now, let's find specified group 
      */
@@ -194,7 +220,7 @@ IsBulletinForMe ( POP *p, const char *bull_name )
      * If user's primary group matches the bulletin's group, we matched
      * it the easy way.
      */
-    if ( my_group == b_group ) {
+    if ( pwd->pw_gid == b_group ) {
         DEBUG_LOG1 ( p, "User's gid matches bulletin's: %s", bull_name );
         return 1;
     }
@@ -280,7 +306,9 @@ int
 pop_bull ( POP *p, struct passwd *pwp )
 {
    char             popBullName    [ 256 ];
+#ifdef POPBULLDIR
    char             oldPopBullName [ 256 ];
+#endif
    FILE            *popBull                     = NULL;
    DIR             *dirp;
    file_list       *list                        = NULL;
@@ -289,31 +317,34 @@ pop_bull ( POP *p, struct passwd *pwp )
    long             bullNumber;
    long             lastBullSent;
    char             buffer [ MAXMSGLINELEN ];
-   int              res;
-   int              bullcount                   = 0;
+   int              res                         = POP_SUCCESS;
+   int              err                         = 0;
    int              save_count;
-   mode_t           my_umask;
+   int              save_visible_count;
+   long             save_drop_size;
 
 #ifdef BULLDB
    datum            bull_count;
    datum            name;
+#else
+   mode_t           my_umask;
 #endif /* BULLDB */
 
    /* 
     * Construct full path name of .popbull file.
     */
 #ifdef POPBULLDIR
-   strncpy  ( popBullName, POPBULLDIR, sizeof(popBullName) );
+   strlcpy  ( popBullName, POPBULLDIR, sizeof(popBullName) );
    if ( popBullName [ strlen(popBullName) -1 ] != '/' )
-       strncat ( popBullName, "/", sizeof(popBullName) - strlen(popBullName) );
-   strncat ( popBullName, p->user,    sizeof(popBullName) - strlen(popBullName) );
-   strncat ( popBullName, ".popbull", sizeof(popBullName) - strlen(popBullName) );
+       strlcat ( popBullName, "/", sizeof(popBullName) );
+   strlcat ( popBullName, p->user,    sizeof(popBullName) );
+   strlcat ( popBullName, ".popbull", sizeof(popBullName) );
    
-   strncpy ( oldPopBullName, pwp->pw_dir, sizeof(oldPopBullName) );
-   strncat ( oldPopBullName, "/.popbull", sizeof(oldPopBullName) - strlen(oldPopBullName) );
+   strlcpy ( oldPopBullName, pwp->pw_dir, sizeof(oldPopBullName) );
+   strlcat ( oldPopBullName, "/.popbull", sizeof(oldPopBullName) );
 #else /* not using POPBULLDIR */
-   strncpy ( popBullName, pwp->pw_dir, sizeof(popBullName) );
-   strncat ( popBullName, "/.popbull", sizeof(popBullName) - strlen(popBullName) );
+   strlcpy ( popBullName, pwp->pw_dir, sizeof(popBullName) );
+   strlcat ( popBullName, "/.popbull", sizeof(popBullName) );
 #endif /* POPBULLDIR */
 
    /* 
@@ -329,7 +360,7 @@ pop_bull ( POP *p, struct passwd *pwp )
    }
 
     while ( ( dp = readdir ( dirp ) ) != NULL ) {
-      if ( !isdigit ( *dp->d_name ) ) 
+      if ( isdigit ( (int) *dp->d_name ) == FALSE ) 
          continue;
       bullNumber = atol ( dp->d_name );
       DEBUG_LOG2 ( p, "Found bulletin #%ld: %s",
@@ -338,7 +369,7 @@ pop_bull ( POP *p, struct passwd *pwp )
        * We found a bulletin.  Insert it if it is for the user.
        */
       if ( BULL_GROUPS == 0 ||
-           IsBulletinForMe ( p, dp->d_name ) 
+           IsBulletinForMe ( p, pwp, dp->d_name ) 
          ) {
         if ( bullNumber > maxBullNumber ) 
             maxBullNumber = bullNumber;
@@ -361,7 +392,7 @@ pop_bull ( POP *p, struct passwd *pwp )
     */
 
 #ifdef BULLDB
-   name.dptr = p->user;
+   name.dptr  = p->user;
    name.dsize = strlen ( p->user ) + 1;
 
 #  ifdef GDBM
@@ -400,7 +431,7 @@ pop_bull ( POP *p, struct passwd *pwp )
            popBull = NULL;
        }
        else
-       if ( !isdigit(*buffer) ) {
+       if ( isdigit ( (int) *buffer ) == FALSE ) {
            DEBUG_LOG2 ( p, "Invalid data in file %s: \"%.128s\"",
                         popBullName, buffer );
            lastBullSent = maxBullNumber - NEWBULLCNT;
@@ -448,13 +479,13 @@ pop_bull ( POP *p, struct passwd *pwp )
 
    popBull = fopen ( popBullName, "r" );
 
-#ifdef POPBULLDIR
+#  ifdef POPBULLDIR
    if ( popBull == NULL ) {
        popBull = fopen ( oldPopBullName, "r" );
        DEBUG_LOG2 ( p, "User's %s file doesn't exist -- trying %s",
                     popBullName, oldPopBullName );
    }
-#endif /* POPBULLDIR */
+#  endif /* POPBULLDIR */
 
    if ( popBull == NULL ) {
        DEBUG_LOG1 ( p, "No .popbull file for %s", p->user );
@@ -486,7 +517,7 @@ pop_bull ( POP *p, struct passwd *pwp )
        popBull = NULL;
    }
    else
-   if ( !isdigit(*buffer) ) {
+   if ( !isdigit ( (int) *buffer ) ) {
        DEBUG_LOG2 ( p, "Invalid data in file %s: \"%.128s\"",
                     popBullName, buffer );
        lastBullSent = maxBullNumber - NEWBULLCNT;
@@ -527,43 +558,47 @@ pop_bull ( POP *p, struct passwd *pwp )
     * If there aren't any new bulletins for this user, return. 
     */
    if ( lastBullSent >= maxBullNumber ) 
-      return POP_SUCCESS;
+       return POP_SUCCESS;
 
    /* 
     * Append the new bulletins to the end of the user's temporary 
     * mail drop. 
     */
 
-   res = POP_SUCCESS;
-   save_count = p->msg_count;
+   res                = POP_SUCCESS;
+   save_count         = p->msg_count;
+   save_visible_count = p->visible_msg_count;
+   save_drop_size     = p->drop_size;
 
    while ( list != NULL ) {
        if ( list->bull_value > lastBullSent ) {
-          res = CopyOneBull ( p, list->bull_value, list->bull_name );
-           if ( res != POP_SUCCESS ) {
-               p->msg_count = save_count;
-               break;
-           }
+            res = CopyOneBull ( p, list->bull_value, list->bull_name );
+            if ( res != POP_SUCCESS ) {
+                break;
+            }
        }
        list = list->next;
    }
 
    if ( res == POP_SUCCESS ) {
+
 #ifdef BULLDB
-       bull_count.dptr  = (char *)&maxBullNumber;
+       bull_count.dptr  = (char *) &maxBullNumber;
        bull_count.dsize = sizeof(maxBullNumber);
-       if ( flock(dbm_dirfno(p->bull_db), LOCK_EX ) == -1 ) {
-           p->msg_count = save_count;
+       err = flock ( dbm_dirfno ( p->bull_db ), LOCK_EX );
+       if ( err == -1 ) {
            pop_log ( p, POP_PRIORITY, HERE, 
-                     "[SYS/TEMP] Bulletin database lock failed" );
-           return POP_FAILURE;
+                     "[SYS/TEMP] Bulletin database lock failed: %s (%d)",
+                     strerror(errno), errno );
+           res = POP_FAILURE;
+           goto Exit;
        }
 #  ifdef GDBM
-       gdbm_store(p->bull_db, name, bull_count, GDBM_REPLACE);
+       gdbm_store ( p->bull_db, name, bull_count, GDBM_REPLACE );
 #  else
-       dbm_store(p->bull_db, name, bull_count, DBM_REPLACE);
+       dbm_store  ( p->bull_db, name, bull_count, DBM_REPLACE );
 #  endif /* GDBM */
-       flock(dbm_dirfno(p->bull_db), LOCK_UN);
+       flock ( dbm_dirfno ( p->bull_db ), LOCK_UN );
 #else /* not BULLDB */
 
        /* 
@@ -578,28 +613,44 @@ pop_bull ( POP *p, struct passwd *pwp )
        umask ( my_umask );
 
        if ( popBull == NULL ) {
-           p->msg_count = save_count;
+            
            pop_log ( p, POP_PRIORITY, HERE, 
                      "Unable to open %s for writing: %s (%d)", 
                      popBullName, strerror(errno), errno );
-           return POP_FAILURE;
+           res = POP_FAILURE;
+           goto Exit;
        }
        if ( fprintf ( popBull, "%ld\n", maxBullNumber ) < 0 ) {
            pop_log ( p, POP_PRIORITY, HERE,
                      "Unable to write to %s: %s (%d)",
                      popBullName, strerror(errno), errno );
-           return POP_FAILURE;
+           res = POP_FAILURE;
+           goto Exit;
        }
        if ( fclose ( popBull ) == EOF ) {
            pop_log ( p, POP_PRIORITY, HERE,
                      "Unable to close %s after writing: %s (%d)",
                      popBullName, strerror(errno), errno );
-           return POP_FAILURE;
+           res = POP_FAILURE;
+           goto Exit;
        }
 #endif /* BULLDB */
-   }
+   } /* res == POP_SUCCESS */
 
-   return ( res );
+Exit:
+   if ( res != POP_SUCCESS ) {
+       DEBUG_LOG6 ( p, "pop_bull returning failure; restoring msg_count "
+                       "from %d to %d; visible from %d to %d; "
+                       "drop size from %ld to %ld",
+                    p->msg_count,         save_count,
+                    p->visible_msg_count, save_visible_count,
+                    p->drop_size,         save_drop_size );
+       p->msg_count         = save_count;
+       p->visible_msg_count = save_visible_count;
+       p->drop_size         = save_drop_size;
+   }
+   
+   return res;
 }
 
 extern int newline; /* lives in pop_dropcopy.c */
@@ -609,6 +660,7 @@ extern int newline; /* lives in pop_dropcopy.c */
  *  temporary maildrop file.
  */
 
+int
 CopyOneBull ( POP *p, long bnum, char *name )
 {
     FILE          *bull;
@@ -618,21 +670,20 @@ CopyOneBull ( POP *p, long bnum, char *name )
     int            nchar;
     int            msg_num;
     int            msg_vis_num          = 0;
-    int            receivedhdrs         = 0;
     int            msg_ends_in_nl       = 0;
     char           bullName [ 256 ];
     MsgInfoList   *mp;               /* Pointer to message info list */
 
 
-    msg_num = ( p->msg_count ) - 1;
+    msg_num = p->msg_count;
     while ( msg_vis_num == 0 && msg_num > 0 ) {
-        mp = &p->mlp [ msg_num-- ];
+        mp = &p->mlp [ --msg_num ];
         if ( mp->visible_num > 0 )
             msg_vis_num = mp->visible_num;
     }
 
     msg_num = p->msg_count;
-    p->msg_count = (((p->msg_count - 1) / ALLOC_MSGS) + 1) * ALLOC_MSGS;
+    p->msg_count = ( ( ( p->msg_count - 1 ) / ALLOC_MSGS ) + 1 ) * ALLOC_MSGS;
 
     sprintf ( bullName, "%s/%s", p->bulldir, name );
     bull = fopen ( bullName, "r" );
@@ -707,15 +758,16 @@ CopyOneBull ( POP *p, long bnum, char *name )
         return POP_FAILURE;
     }
 
-    mp->length += strlen(buffer);
-    p->drop_size += strlen(buffer);
+    mp->length   += strlen ( buffer );
+    p->drop_size += mp->length;
     mp->lines++;
 
-    DEBUG_LOG5 ( p, "Bull msg %lu (as %d [vis: %d]) being added to list, offset %d: %s",
-                bnum, mp->number, mp->visible_num, mp->offset, name );
+    DEBUG_LOG5 ( p, "Bull msg %lu (as %d [vis: %d]) being added to "
+                    "list, offset %lu: %s",
+                 bnum, mp->number, mp->visible_num, mp->offset, name );
     sprintf ( buffer, "%ld.%03d\n", timestamp, sequence++ );
 
-    mp->uidl_str = (char *)strdup(buffer);
+    mp->uidl_str = (char *) strdup ( buffer );
 
     first_line = 0;
 
