@@ -6,8 +6,43 @@
  *
  * Revisions: 
  *
- * 05/17/00  [rg]
- *         - Tweak mail lock refresh algorithm to avoid performance hit.
+ * 10/04/00  [rcg]
+ *         - Allow Content-Length: header to be CONTLEN_FUZZ greater than
+ *           reality.
+ *
+ * 08/17/00  [rcg]
+ *         - Adjusted bulldb open algorithm to use usleep() if available
+ *           and thus wait for very short periods, with randomness.
+ *         - Added support for bulldb-nonfatal option, to allow session to
+ *           proceed without bulletins.
+ *
+ * 07/05/00  [rcg]
+ *         - Print maillock error string.
+ *         - Be a little more patient locking spool.
+ *
+ * 06/27/00  [rcg]
+ *         - Fix UID generation when OLD_STYLE_UIDL set to match 2.53.
+ *
+ * 06/16/00  [rcg]
+ *         - Don't check for old spools in old location unless HASH_SPOOL
+ *           or HOMEDIRMAIL set, and DONT_CHECK_OLD_SPOOL not set.
+ *         - Deleted some unused variables.
+ *
+ * 06/08/00  [rcg]
+ *         - UIDs now deterministic when NO_STATUS set.
+ *         - Removed switch for old-style locking.
+ *         - Added switch (OLD_STYLE_UIDL) to encode UIDLs in old
+ *           (pre-3.x) style.
+ *
+ * 06/05/00  [rcg]
+ *         - Applied patches from Jeffrey C. Honig at BSDI.
+ *
+ * 05/16/00  [rg]
+ *         - We now check if we need to refresh the mail lock every
+ *           p->check_lock_refresh messages.
+ *         - When '-Z' run-time switch used, we revert to old (pre-3.x)
+ *           locking.  This is dangerous and will likely be removed prior
+ *           to release.
  *
  * 04/28/00  [rg]
  *         - Avoid being fooled by a spoofed 'From ' line that follows
@@ -100,10 +135,10 @@
 #include <errno.h>
 #include <sys/errno.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
 #include <flock.h>
-#include <time.h>
 
 #include "config.h"
 
@@ -128,6 +163,8 @@
 #include <sys/file.h>
 #include <pwd.h>
 
+#include <time.h>
+
 #include "genpath.h"
 #include "popper.h"
 #include "misc.h"
@@ -139,10 +176,13 @@
 #  define UW_KLUDGE 0
 #endif /* CHECK_UW_KLUDGE */
 
-
 #define MIN_UIDL_LENGTH 5     /* There is no particular reason to have this */
 #define MAX_UIDL_LENGTH 70
 
+#ifndef   CONTLEN_FUZZ
+#  define CONTLEN_FUZZ 25
+#endif /* CONTLEN_FUZZ */
+
 /* This macro comes from Mark Crispin's c-client code */
 
 /* Validate line
@@ -210,6 +250,23 @@
  */
 
 
+/*
+ * Prototypes
+ */
+int     isfromline(char *cp);
+char   *base80(char *output, unsigned char *digest);
+char   *hexit(char *output, unsigned char *digest);
+char   *encode_uid_hash(char *output, unsigned char *digest);
+int     init_dropinfo(POP *p, char *fname, FILE *fdesc, time_t time_locked);
+int     do_drop_copy(POP *p, int mfd, time_t time_locked);
+int     pop_dropcopy(POP *p, struct passwd *pwp);
+char   *mfgets(char *s, int size, FILE *stream);
+void    doze(POP *p, int atmpt);
+
+
+/*
+ * Globals
+ */
 int newline  = 1;   /* set to 1 when preceding line was only a newline */
 int isOflow  = 0;   /* set to 1 when current line exceeds input buffer */
 int wasOflow = 0;   /* set to 1 when previous line exceeded input buffer */
@@ -272,7 +329,7 @@ char        *cp;
 char *
 base80 ( char *output, unsigned char *digest )
 {
-  unsigned long v, n, j;
+  unsigned long v, j;
   int i, k;
 
   for ( i = 0; i < 16; ) {
@@ -285,7 +342,7 @@ base80 ( char *output, unsigned char *digest )
       v = (v - j)/80;
       *output = 0x21 + j;
       /* 
-       * Avoid . altgother to avoid having to stuff it 
+       * Avoid '.' altgother to avoid having to stuff it 
        */
       if ( *output == '.' )
         *output = '~';
@@ -297,6 +354,37 @@ base80 ( char *output, unsigned char *digest )
 }
 
 
+/*
+ * Old-style (pre-3.x) encoding of MD5 digests for UID strings was simple hex
+ */
+char *
+hexit ( char *output, unsigned char *digest )
+{
+    int   i;
+
+
+    for ( i = 0; 
+          i < DIG_SIZE; 
+          i++, output += 2 ) {
+        sprintf ( output, "%02x", digest[i] );
+    }
+
+    *output = '\0';
+    return ( output );
+}
+
+
+char *
+encode_uid_hash ( char *output, unsigned char *digest )
+{
+#ifdef  OLD_STYLE_UIDL
+    return hexit  ( output, digest );
+#else   /* not OLD_STYLE_UIDL */
+    return base80 ( output, digest );
+#endif  /* OLD_STYLE_UIDL */
+}
+
+
 /* 
  * Build the Msg_info_list from the temp_drop in case the previous session
  * failed to truncate. init_dropinfo and do_drop_copy differ in that
@@ -321,7 +409,7 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
                                              * computation */
     MD5_CTX         mdContext;
     unsigned char   digest[16];
-    long            unique_num = time(0);
+    long            unique_num = time_locked;
     int             uw_hint    = 0;
 
     DEBUG_LOG1 ( p, "DROPINFO Checking file %s", fname );
@@ -372,6 +460,9 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
     p->msg_count      = ALLOC_MSGS;
 
 
+    if ( DEBUGGING && time_locked == 0 )
+        pop_log ( p, POP_NOTICE, HERE, "init_dropinfo called with time_locked == 0" );
+
 #ifdef NULLKLUDGE
     {
     /* 
@@ -390,17 +481,19 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
         
         /*
          * It is very important to refresh the mail lock on an
-         * interval between one and three minutes.  
+         * interval between one and three minutes.  To avoid
+         * calling time(0) too often, we check every x msgs.
          */
-        if ( msg_num % 500 == 0 &&
-             time(0) - time_locked > 60 ) {
-            Qtouchlock();
+        if ( msg_num % p->check_lock_refresh == 0 &&
+             time(0) - time_locked >= LOCK_REFRESH_INTERVAL ) {
+            Qtouchlock ( HERE );
             time_locked = time(0);
         }
 
-        if ( ( content_nchar >= content_length ) &&
-             ( p->mmdf_separator ? !strcmp(p->mmdf_separator, buffer) :
-                                   isfromline(buffer) ) ) {
+        if ( ( content_nchar >= content_length ||
+               content_length - content_nchar <= CONTLEN_FUZZ )
+            && ( p->mmdf_separator ? !strcmp(p->mmdf_separator, buffer)
+                                   : isfromline(buffer) ) ) {
             /* 
              * ---- A message boundary ----- 
              *
@@ -425,12 +518,19 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
             MD5Init   ( &mdContext );
             MD5Update ( &mdContext, (unsigned char *)buffer,
                         strlen(buffer) );
+
+#ifndef NO_STATUS
             /* 
-             * Include a unique number in case everything else is not 
+             * Include a unique number in case everything else is not.
+             *
+             * We don't do this if NO_STATUS is set, because then the UIDs
+             * have to be deterministic so they stay the same when
+             * recalculated each session.
              */
             unique_num++;
             MD5Update ( &mdContext, (unsigned char *)&unique_num, 
                         sizeof(long) );
+#endif /* not NO_STATUS */
 
             if ( !inheader ) {
                 if ( ++msg_num > p->msg_count ) {
@@ -448,7 +548,7 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
                 }
 
                 if ( msg_num != 1 ) {
-                    DEBUG_LOG5 ( p, "Msg %d uidl '%s' at offset %d is %d "
+                    DEBUG_LOG5 ( p, "Msg %d uidl '%s' at offset %ld is %ld "
                                     "octets long and has %u lines.",
                                  mp->number, mp->uidl_str, mp->offset, 
                                  mp->length, mp->lines );
@@ -474,7 +574,7 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
             mp->retr_flag  = FALSE;
             mp->orig_retr_state = FALSE;
             mp->uidl_str   = "\n";
-            DEBUG_LOG2 ( p, "Msg %d being added to list %x", mp->number, p->mlp );
+            DEBUG_LOG2 ( p, "Msg %d being added to list %p", mp->number, p->mlp );
             inheader       = 1;
             uidl_found     = 0;
             content_nchar  = 0;
@@ -496,13 +596,15 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
                 
                 if ( !uidl_found ) {
                     char    *cp;
-                    int     i;
 
+#ifndef   OLD_STYLE_UIDL
                     MD5Update ( &mdContext, (unsigned char *)frombuf, 
                                 strlen(frombuf) );
+#endif /* OLD_STYLE_UIDL */
+
                     MD5Final ( digest, &mdContext );
                     cp = mp->uidl_str = (char *)malloc((DIG_SIZE * 2) + 2);
-                    cp = base80 ( cp, digest );
+                    cp = encode_uid_hash ( cp, digest );
                     *cp++ = '\n';
                     *cp   = '\0';
                     
@@ -628,13 +730,11 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
  *  Now, the copy and info collection are done at the same time.
  *
  *  Parameters:
- *     p:           Pointer to POP structure.
- *     mfd:         File descriptor for spool file (p->drop_name is name).
- *     dfd:         File descriptor for temp drop  (p->temp_drop is name).
- *     time_locked  time(0) when maillock acquired.
+ *     p:         Pointer to POP structure.
+ *     mfd:       File descriptor for spool file (p->drop_name is name).
  */
 int
-do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked ) 
+do_drop_copy ( POP *p, int mfd, time_t time_locked ) 
 {
     char            buffer  [ MAXLINELEN ];      /*  Read buffer */
     char            frombuf [ MAXLINELEN ];      /*  To save From Envelope */
@@ -649,14 +749,17 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
     int             content_length, content_nchar, cont_len;
     MD5_CTX         mdContext;
     unsigned char   digest[16];
-    int             mangled_one;
     FILE           *mail_drop;                   /*  Streams for fids */
-    long            unique_num = time(0);
+    long            unique_num = time_locked;
     int             uw_hint    = 0;
 
 
     DEBUG_LOG1 ( p, "DROPCOPY: Reading the mail drop (p->msg_count = %i).",
                  p->msg_count );
+
+    if ( DEBUGGING && time_locked == 0 )
+        pop_log ( p, POP_NOTICE, HERE, "do_drop_copy called with time_locked == 0" );
+
     /*  
      * Acquire a stream pointer for the maildrop 
      */
@@ -716,7 +819,6 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
     content_length    = 0;
     content_nchar     = 0;
     cont_len          = 0;
-    mangled_one       = 0;
     p->msg_count      = (((p->msg_count - 1) / ALLOC_MSGS) + 1) * ALLOC_MSGS;
 
 #ifdef NULLKLUDGE
@@ -735,11 +837,12 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
         
         /*
          * It is very important to refresh the mail lock on an
-         * interval between one and three minutes. 
+         * interval between one and three minutes.  To avoid
+         * calling time(0) too often, we check every x msgs.
          */
-        if ( msg_num % 500 == 0 &&
-             time(0) - time_locked > 60 ) {
-            Qtouchlock();
+        if ( msg_num % p->check_lock_refresh == 0 &&
+             time(0) - time_locked >= LOCK_REFRESH_INTERVAL ) {
+            Qtouchlock ( HERE );
             time_locked = time(0);
         }
 
@@ -758,11 +861,10 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
         /* 
          * End of previous message 
          */
-        
-        if ( ( content_nchar >= content_length ) &&
-             ( p->mmdf_separator ? !strcmp ( p->mmdf_separator, buffer )
-                                 : isfromline ( buffer )
-             ) 
+        if ( ( content_nchar >= content_length ||
+               content_length - content_nchar <= CONTLEN_FUZZ )
+            && ( p->mmdf_separator ? !strcmp(p->mmdf_separator, buffer)
+                                   : isfromline(buffer) ) 
            ) {
             /*
              * === Matched separator, tie off last message, start new one ===
@@ -783,11 +885,19 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
              */
             MD5Init   ( &mdContext );
             MD5Update ( &mdContext, (unsigned char *)buffer, strlen(buffer) );
+
+#ifndef NO_STATUS
             /* 
-             * Include a unique number in case everything else is not 
+             * Include a unique number in case everything else is not.
+             *
+             * We don't do this if NO_STATUS is set, because then the UIDs
+             * have to be deterministic so they stay the same when
+             * recalculated each session.
              */
             unique_num++;
-            MD5Update ( &mdContext, (unsigned char *)&unique_num, sizeof(long) );
+            MD5Update ( &mdContext, (unsigned char *)&unique_num, 
+                        sizeof(long) );
+#endif /* not NO_STATUS */
 
             if ( !inheader ) {
                 if ( ++msg_num > p->msg_count ) {
@@ -804,7 +914,7 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
                     mp = p->mlp + msg_num - 2;
                 } /* ++msg_num > p->msg_count */
             if ( msg_num != 1 )
-                DEBUG_LOG5 (p, "Msg %d uidl '%s' at offset %d is %d octets long"
+                DEBUG_LOG5 (p, "Msg %d uidl '%s' at offset %ld is %ld octets long"
                                " and has %u lines.",
                             mp->number, mp->uidl_str, mp->offset, mp->length,
                             mp->lines );
@@ -858,13 +968,15 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
 
                 if ( !uidl_found ) {
                     char *cp;
-                    int  i;
 
+#ifndef   OLD_STYLE_UIDL
                     MD5Update ( &mdContext, (unsigned char *)frombuf,
                                 strlen(frombuf) );
+#endif /* OLD_STYLE_UIDL */
+
                     MD5Final  ( digest, &mdContext );
                     cp = mp->uidl_str = (char *)malloc((DIG_SIZE * 2) + 2);
-                    cp = base80 ( cp, digest );
+                    cp = encode_uid_hash ( cp, digest );
                     *cp++ = '\n';
                     *cp   = '\0';
                     DEBUG_LOG3 ( p, "UID not found; generated UID(%d): %s [%d]",
@@ -996,20 +1108,18 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
     p->msg_count = msg_num;
     p->visible_msg_count = visible_msg_num;
 
-#ifdef DEBUG
-    if ( p->debug && msg_num > 0 ) {
+    if ( DEBUGGING && p->debug && msg_num > 0 ) {
         register int    i;
         
         for ( i = 0, mp = p->mlp; i < p->msg_count; i++, mp++ )
             pop_log ( p, POP_DEBUG, HERE,
-                      "Msg %d (%d) uidl '%s' at offset %d is %d octets long"
+                      "Msg %d (%d) uidl '%s' at offset %ld is %ld octets long"
                       "%.*s and has %u lines.",
                       mp->number, mp->visible_num, mp->uidl_str, 
                       mp->offset, mp->length,
                       (mp->hide_flag ? 9 : 0), " (hidden)",
                       mp->lines );
     }
-#endif
 
     if ( fflush ( p->drop ) == EOF )
         return pop_msg ( p, POP_FAILURE, HERE, 
@@ -1024,6 +1134,7 @@ do_drop_copy ( POP *p, int mfd, int dfd, time_t time_locked )
  *  save a stream pointer for it.
  */
 
+int
 pop_dropcopy ( p, pwp )
 POP        *   p;
 struct passwd   * pwp;
@@ -1032,15 +1143,15 @@ struct passwd   * pwp;
                                                         the user's maildrop */
     int                     dfd;                    /*  File descriptor for 
                                                         the SERVER maildrop */
-    FILE                   *tf;                     /*  The temp file */
-    int                     tfn;            
-    char                    buffer [ MAXLINELEN ];  /*  Read buffer */
     OFF_T                   offset = 0;             /*  Old/New boundary */
     int                     nchar;                  /*  Bytes written/read */
     struct stat             mybuf;                  /*  For fstat() */
-    char                   *ptr;
     int                     rslt;
-    time_t                  time_locked;
+    time_t                  time_locked = 0;
+
+#ifdef DO_TIMING
+    time_t                  my_timer = time(0);
+#endif /* DO_TIMING */
 
 
     if ( genpath ( p->user, 
@@ -1052,6 +1163,8 @@ struct passwd   * pwp;
         return ( pop_msg ( p, POP_FAILURE, HERE, 
                           "[SYS/TEMP] Unable to get temporary drop name" ) );
 
+#if ( ( defined(HASH_SPOOL) || defined(HOMEDIRMAIL) ) && !defined(DONT_CHECK_OLD_SPOOL) )
+
     /*  
      *  Get name of temporary maildrop into which to copy the updated 
      *  maildrop. 
@@ -1081,49 +1194,71 @@ struct passwd   * pwp;
             return ( pop_msg ( p, POP_FAILURE, HERE,
                                "[SYS/TEMP] Unable to get temporary drop name" ) );
     }
+
+#else  /* not using HASH_SPOOL or HOMEDIRMAIL, or set DONT_CHECK_OLD_SPOOL */
+
+    if ( genpath ( p->user, 
+                   p->temp_drop, 
+                   sizeof(p->temp_drop),
+                   GNPH_POP,
+                   (p->debug ? p->trace : NULL),
+                   p->debug ) < 0 )
+        return ( pop_msg ( p, POP_FAILURE, HERE,
+                           "[SYS/TEMP] Unable to get temporary drop name" ) );
+
+#endif /* ( ( defined(HASH_SPOOL) || defined(HOMEDIRMAIL) ) && !defined(DONT_CHECK_OLD_SPOOL) ) */
     
     DEBUG_LOG1 ( p, "Temporary maildrop name: '%s'", p->temp_drop );
 
 #ifdef BULLDB
-#  define SLEEP_INTERVAL 1 /* how long to wait between tries */
-#  define MAX_ATTEMPTS  10 /* when to give up */
-    if (p->bulldir) {
+
+#  ifdef HAVE_USLEEP
+#    define MAX_ATTEMPTS   100 /* when to give up when sleeping for milliseconds */
+#  else
+#    define MAX_ATTEMPTS    10 /* when to give up when sleeping for seconds */
+#  endif /* HAVE_USLEEP */
+
+    if ( p->bulldir != NULL ) {
         char bull_db [ MAXLINELEN ];
         int  tries     = 0;
-        int  db_err    = 0;
-        int  try_again = 1;
+        BOOL try_again = TRUE;
 
 #  ifdef BULLDBDIR
-        sprintf(bull_db, "%s/bulldb", BULLDBDIR);
+        sprintf ( bull_db, "%s/bulldb", BULLDBDIR  );
 #  else /* BULLDBDIR not defined, use bull dir */
-        sprintf(bull_db, "%s/bulldb", p->bulldir);
+        sprintf ( bull_db, "%s/bulldb", p->bulldir );
 #  endif /* BULLDBDIR */
 
         p->bull_db = NULL;
     
         while ( try_again ) {
 #  ifdef GDBM
-            p->bull_db = gdbm_open (bull_db, 512, GDBM_WRCREAT, 0600, 0);
+            p->bull_db = gdbm_open ( bull_db, 512, GDBM_WRCREAT, 0600, 0 );
 #  else /* not GDBM */
-            p->bull_db = dbm_open (bull_db, O_RDWR | O_CREAT, 0600);
+            p->bull_db = dbm_open  ( bull_db, O_RDWR | O_CREAT,  0600    );
 #  endif /* GDBM */
             if ( p->bull_db == NULL && errno == EAGAIN
                                     && tries++ <= MAX_ATTEMPTS ) {
                 DEBUG_LOG3 ( p, "Bulletin database '%s' unavailable on "
                                 "try #%d (user %s); sleeping",
                             bull_db, tries, p->user );
-                sleep ( tries * SLEEP_INTERVAL );   /* Sleep between retries */
+                doze ( p, tries );   /* Sleep between retries */
             } else {
-                try_again = 0;
+                try_again = FALSE;
             }
         } /* while loop */
 
         if ( p->bull_db == NULL ) {
             pop_log (p, POP_PRIORITY, HERE, "gdbm_open failed: %s (%d)",
-                    sys_errlist[errno], errno );
-            return ( pop_msg ( p, POP_FAILURE, HERE, 
-                               "[SYS/TEMP] Unable to open Bulletin "
-                               "database; contact your administrator" ) );
+                        sys_errlist[errno], errno );
+            if ( p->bulldb_nonfatal == FALSE )
+                return ( pop_msg ( p, POP_FAILURE, HERE, 
+                                   "[SYS/TEMP] Unable to open Bulletin "
+                                   "database; contact your administrator" ) );
+            else
+                DEBUG_LOG2 ( p, "Unable to open bulldb '%s' for user '%s' "
+                                "but bulldb_nonfatal set",
+                             bull_db, p->user );
         } else {
             DEBUG_LOG2 ( p, "Opened Bulletin database '%s' (checking user '%s')",
                         bull_db, p->user );
@@ -1132,11 +1267,12 @@ struct passwd   * pwp;
     } /* if (p->bulldir) */
 #endif /* BULLDB */
 
-#ifdef HASH_SPOOL
+#if ( defined(HASH_SPOOL) && !defined(DONT_CHECK_HASH_SPOOL_DIR) )
     { /* HASH_SPOOL block */
     /* 
      * Make sure the path to the spool file exists 
      */
+     char    buffer [ MAXLINELEN ];
      mode_t  my_umask;
      mode_t  spool_mode;
      mode_t  temp_mode = 0x1C0;
@@ -1270,9 +1406,11 @@ struct passwd   * pwp;
     umask ( my_umask );
     DEBUG_LOG1 ( p, "umask now %04o", my_umask );
     } /* HASH_SPOOL block */
-#endif /* HASH_SPOOL */
+#endif /* ( defined(HASH_SPOOL) && !defined(DONT_CHECK_HASH_SPOOL_DIR) ) */
 
-    /* Here we work to make sure the user doesn't cause us to remove or
+
+    /*
+     * Here we work to make sure the user doesn't cause us to remove or
      * write over existing files by limiting how much work we do while
      * running as root.
      */
@@ -1289,14 +1427,19 @@ struct passwd   * pwp;
     /* 
      * Now we run as the user. 
      */
-    (void) setgid ( (GID_T)pwp->pw_gid );
-    /* 
-     * Set the supplementary groups list 
-     */
-    (void) setgroups ( 1, (GID_T *)&pwp->pw_gid ); 
-    (void) setuid ( (UID_T)pwp->pw_uid );
+#if defined(__bsdi__) && _BSDI_VERSION >= 199608
+    (void) setsid();
+    (void) setusercontext ( p->class, pwp, pwp->pw_uid,
+                            LOGIN_SETALL & ~(LOGIN_SETPATH|LOGIN_SETUMASK ) );
+#else /* everything except BSDI after 1996 */
+    (void) setgid ( (GID_T) pwp->pw_gid );
+    (void) setgroups ( 1, (GID_T *) &pwp->pw_gid ); /* Set the supplementary groups list */
+    (void) setuid ( (UID_T) pwp->pw_uid );
+#endif /* __bsdi__) && _BSDI_VERSION >= 199608 */
 
-    DEBUG_LOG2 ( p, "uid = %d, gid = %d", getuid(), getgid() );
+    DEBUG_LOG2 ( p, "uid = %lu, gid = %lu", 
+                 (long unsigned) getuid(), 
+                 (long unsigned) getgid() );
 
     dfd = open ( p->temp_drop, O_RDWR | O_CREAT, 0600 );
     if ( dfd == -1 ) {
@@ -1305,11 +1448,15 @@ struct passwd   * pwp;
                   "'%s': %s (%d)",
                   p->temp_drop, STRERROR(errno), errno );
         return pop_msg ( p, POP_FAILURE, HERE,
-                         "[SYS/TEMP] Failed to create %s with uid %d, gid %d. "
-                         "Change permissions.",
-                          p->temp_drop, pwp->pw_uid, pwp->pw_gid );
+                         "[SYS/TEMP] Failed to create %s with uid %lu, "
+                         "gid %lu.  Change permissions.",
+                          p->temp_drop,
+                          (long unsigned) pwp->pw_uid, 
+                          (long unsigned) pwp->pw_gid );
     }
 
+    DEBUG_LOG2 ( p, "Opened temp drop %s (%d)", p->temp_drop, dfd );
+
     fstat ( dfd, &mybuf );
     if ( mybuf.st_uid != pwp->pw_uid ) {
         close  ( dfd );
@@ -1349,7 +1496,7 @@ struct passwd   * pwp;
      * Lock the temporary maildrop.  Locking is just to check for single
      * pop session per user.
      */
-    if ( flock ( dfd, LOCK_EX|LOCK_NB ) == -1 ) {
+    if ( flock ( dfd, LOCK_EX | LOCK_NB ) == -1 ) {
         switch ( errno ) {
             case EWOULDBLOCK:
                 close ( dfd );
@@ -1392,6 +1539,9 @@ struct passwd   * pwp;
                          "[SYS/TEMP] Cannot assign stream for %s: %s (%d)",
                          p->temp_drop, STRERROR(errno), errno );
     }
+
+    DEBUG_LOG1 ( p, "Set p->drop to stream for %d", dfd );
+
     /*
      * Allocate memory for message information structures and this is
      * not deleted since a failure, for some reason in this function
@@ -1411,7 +1561,7 @@ struct passwd   * pwp;
     if ( mybuf.st_size != 0 ) { /* Mostly this is for regular mode. */
         DEBUG_LOG2 ( p, "Temp drop %s not empty (%lu octets)", 
                      p->temp_drop, mybuf.st_size );
-        if ( init_dropinfo ( p, p->temp_drop, p->drop, time_locked ) != POP_SUCCESS ) {
+        if ( init_dropinfo ( p, p->temp_drop, p->drop, time(0) ) != POP_SUCCESS ) {
             /* Occurs on temp_drop corruption */
             flock ( dfd, LOCK_UN );
             close ( dfd );
@@ -1430,13 +1580,14 @@ struct passwd   * pwp;
      */
 
     DEBUG_LOG0 ( p, "Getting mail lock" );
-    rslt = Qmaillock ( p->user, 1,  p->trace, p->debug );
+    rslt = Qmaillock ( p->user, 2,  p->trace, HERE, p->debug );
     if ( rslt != 0 ) {
         flock ( dfd, LOCK_UN );
         close ( dfd );
         return pop_msg ( p, POP_FAILURE, HERE, 
-                         "[SYS/TEMP] maillock error %d: '%s'", 
-                         rslt, p->drop_name );
+                         "[SYS/TEMP] maillock error '%s' (%d) on '%s': %s (%d)", 
+                         Qlockerr(rslt), rslt, p->drop_name, 
+                         strerror(errno), errno );
     }
     time_locked = time(0);
 
@@ -1454,17 +1605,19 @@ struct passwd   * pwp;
             flock ( dfd, LOCK_UN );
             close ( dfd );
             close ( mfd );
-            Qmailunlock();
+            Qmailunlock ( HERE );
             return pop_msg ( p, POP_FAILURE, HERE, 
                              "[SYS/TEMP] flock: '%s': %s (%d)", 
                              p->drop_name, STRERROR(errno), errno );
         }
 
+        DEBUG_LOG2 ( p, "Opened spool %s (%d)", p->drop_name, mfd );
+
         if ( !p->server_mode ) {
             /*
              * New routine to copy and get dropbox info all at the same time 
              */
-            nchar = do_drop_copy ( p, mfd, dfd, time_locked );
+            nchar = do_drop_copy ( p, mfd, time_locked );
 
             if ( nchar != POP_SUCCESS ) {
                 /* 
@@ -1506,6 +1659,11 @@ struct passwd   * pwp;
                           p->drop_name, STRERROR(errno), errno );
                 goto bailout;
             }
+
+            DEBUG_LOG2 ( p, "Server mode: set p->hold to temp drop (%d) "
+                            "and p->drop to stream for spool (%d)",
+                         fileno(p->hold), mfd );
+
             if ( init_dropinfo ( p, p->drop_name, p->drop, time_locked ) != POP_SUCCESS )
                 goto bailout;
         }
@@ -1518,13 +1676,17 @@ struct passwd   * pwp;
          * copy back to original mail box.
          */
         if ( p->server_mode ) {
-            p->server_mode = 0; 
+            p->server_mode = 0;
         }
         DEBUG_LOG3 ( p, "Unable to open maildrop %s: %s (%d)", 
                      p->drop_name, (errno < sys_nerr) ? sys_errlist[errno] : "", errno );
     }
 
-    if ( p->bulldir ) {
+    if ( p->bulldir    != NULL
+#ifdef BULLDB
+         && p->bull_db != NULL
+#endif /* BULLDB */
+       ) {
         /* 
          * Recalculate offset 
          */
@@ -1551,50 +1713,57 @@ struct passwd   * pwp;
         }
     }
 #ifdef BULLDB
+    if ( p->bull_db != NULL ) {
+
 #  ifdef GDBM
-    gdbm_close ( p->bull_db );
+        gdbm_close ( p->bull_db );
 #  else /* not GDBM */
-    dbm_close ( p->bull_db );
+        dbm_close ( p->bull_db );
 #  endif /* GDBM */
+
     DEBUG_LOG0 ( p, "Closed Bulletin database" );
+    }
 #endif /* BULLDB */
 
     fseek ( p->drop, 0L, SEEK_END );
     p->spool_end = ftell ( p->drop );
-    DEBUG_LOG3 ( p, "Temp drop contains %lu (%lu visible) messages in %lu octets", 
+    DEBUG_LOG3 ( p, "Temp drop contains %u (%u visible) messages in %lu octets", 
                  p->msg_count, p->visible_msg_count, p->spool_end );
 
-#ifdef DEBUG
-    if ( p->debug && p->msg_count > 0 ) {
+    if ( DEBUGGING && p->debug && p->msg_count > 0 ) {
         register int    i;
-        MsgInfoList *mp; 
+        MsgInfoList    *mp; 
                            
         for ( i = 0, mp = p->mlp; i < p->msg_count; i++, mp++ )
             pop_log ( p, POP_DEBUG, HERE,
-                      "Msg %d (%d) uidl '%s' at offset %d is %d octets "
-                      "long%.*s and has %u lines.",
+                      "Msg %d (%d) uidl '%s' at offset %ld is %ld octets "
+                      "long%.*s and has %d lines.",
                       mp->number, mp->visible_num, 
                       mp->uidl_str, mp->offset, mp->length,
                       (mp->hide_flag ? 9 : 0), " (hidden)",
                       mp->lines );
     }
-#endif /* DEBUG */
-    Qmailunlock();
+
+    Qmailunlock ( HERE );
 
     if ( p->server_mode )
         flock ( mfd, LOCK_UN );
 
+#ifdef DO_TIMING
+    p->init_time = time(0) - my_timer;
+#endif /* DO_TIMING */
+
     return ( POP_SUCCESS );
 
 bailout:
     DEBUG_LOG0 ( p, "--- bailing out ---" );
-    Qmailunlock();
+    Qmailunlock ( HERE );
 
 #ifndef KEEP_TEMP_DROP
     if ( fstat ( dfd, &mybuf ) == 0 && mybuf.st_size == 0 ) {
         unlink ( p->temp_drop ); /* didn't get to use it */
         DEBUG_LOG1 ( p, "Removed empty temp drop: %s", 
-                     p->temp_drop )
+                     p->temp_drop );
     }
 #endif /* KEEP_TEMP_DROP */
 
@@ -1656,3 +1825,39 @@ FILE *stream;
     return s;
 }
 
+/*
+ * Suspends execution for a small amount of time.
+ */
+void doze ( POP *p, int atmpt )
+{
+#define MAX_USEC        500000 /* try not to sleep more than about .5 seconds */
+
+#ifdef HAVE_USLEEP
+    static BOOL inited = FALSE;
+    int         r;
+    long        t;
+
+    /*
+     * We use rand() because it's reportedly faster, and we're not as
+     * concerned with randomness here.  (We could use lower bits of
+     * tv_usec from gettimeofday here and it would probably be better.)
+     */
+    if ( inited == FALSE ) {
+        inited = TRUE;
+        srand ( time ( 0 ) );
+    }
+    
+    r = rand();
+    t = ( r % MAX_USEC ) + atmpt;
+    DEBUG_LOG1 ( p, "...dozing for %ld milliseconds", (t + 500) / 1000 );
+    usleep ( t );
+
+#else /* don't have USLEEP */
+    int t = ( atmpt == 0 ? 2 : atmpt + 1 ) / 2;
+
+    DEBUG_LOG1 ( p, "...dozing for %d seconds", t );
+    sleep ( t );
+
+#endif /* HAVE_USLEEP */
+}
+
