@@ -53,5 +53,5 @@ POP     *   p;
     /*  
      * Otherwise assume NOOP 
      */
-    return ( pop_msg ( p, POP_SUCCESS, HERE, NULL ) );
+    return ( pop_msg ( p, POP_SUCCESS, HERE, "Command ignored" ) );
 }
