@@ -16,19 +16,25 @@
  *  Header file for the POP programs
  */
 
+#include "config.h"
+
+
 #ifndef _POPPER_H
 #  define _POPPER_H
 
 #include <sys/types.h>
+#include <pwd.h>
 #include <md5.h>
 
 #define BINMAIL_IS_SETGID 1
 
-#ifdef POPSCO
-#  include <sys/syslog.h>
-#else
+#ifdef HAVE_SYSLOG_H
 #  include <syslog.h>
-#endif /* POPSCO */
+#endif /* HAVE_SYSLOG_H */
+
+#ifdef HAVE_SYS_SYSLOG_H
+#  include <sys/syslog.h>
+#endif /* HAVE_SYS_SYSLOG_H */
 
 #ifdef ISC
 #  include <sys/fcntl.h>
@@ -43,14 +49,14 @@
 #  endif /* (defined(BSD) && (BSD >= 199103)) */
 #endif /* HAVE_SYS_PARAM_H */
 
-#if defined(BSDI) && !defined(BSD44_DBM)
-#  define BSD44_DBM
-#endif /* BSDI */
+#if defined(__bsdi__)
+#  if   _BSDI_VERSION >= 199608
+#    include <login_cap.h>
+#  endif /* _BSDI_VERSION >= 199608 */
+#endif /* __bsdi__ */
 
 #include <sys/socket.h> /* this needs to be after system .h files */
 
-#include "config.h"
-
 #ifdef BULLDB
 #  undef    DBM     /* used by mts.c and ndbm.h */
 #  ifdef GDBM
@@ -61,9 +67,9 @@
 #endif /* BULLDB */
 
 
-
 #include "version.h"
 #include "banner.h"
+#include "utils.h" /* for TRUE, FALSE, and BOOL */
 
 
 #ifdef SCRAM
@@ -74,8 +80,6 @@
 #define NULLCP          ((char *) 0)
 #define SPACE           32
 #define TAB             9
-#define TRUE            1
-#define FALSE           0
 #define NEWLINE         '\n'
 
 #define MAXUSERNAMELEN  65
@@ -85,7 +89,34 @@
 #define MAXCMDLEN       4
 #define MAXPARMCOUNT    5
 #define MAXPARMLEN      10
-#define ALLOC_MSGS  20
+#define ALLOC_MSGS      20
+
+#ifndef MAXHOSTNAMELEN
+#  define MAXHOSTNAMELEN    64
+#endif /* not MAXHOSTNAMELEN */
+
+#ifdef POPSCO
+#  include <sys/security.h>
+#  include <sys/audit.h>
+#  include <prot.h>
+#  define VOIDSTAR
+#  ifdef SCOR5
+#    undef  VOIDSTAR
+#    define VOIDSTAR      (void (*)(int))
+#  endif
+#else /* not POPSCO */
+#  if ( defined(AIX) || defined (HPUX) || defined(IRIX) )
+#    define VOIDSTAR      (void (*)(int))
+#  else
+#    define VOIDSTAR      (void *)
+#   endif
+#endif /* POPSCO */
+
+#if defined(SCOR5) || defined(AUX) || defined(HPUX)
+#  define SIGPARAM int foo
+#else
+#  define SIGPARAM 
+#endif /* SCOR5 or AUX or HPUX */
 
 #ifndef POP_FACILITY
 #  if defined(OSF1) || defined(LINUX)
@@ -128,6 +159,35 @@ typedef enum {
 #endif /* DEBUG */
 
 
+/*
+ * Some OSes don't have srandom, but do have srand
+ */
+#ifndef   HAVE_SRANDOM
+#  include <stdlib.h>
+#  define srandom srand
+#  define  random  rand
+#endif /* HAVE_SRANDOM */
+
+/*
+ * Some OSes don't have EDQUOT defined.  If so, define it
+ * to be something unlikely to be used.
+ */
+#include <errno.h>
+#ifndef EDQUOT
+#  include <limits.h>
+#  define EDQUOT  INT_MIN /* -2147483647-1 on 32-bit systems */
+#endif /* no EDQUOT */
+
+/*
+ * In standalone mode we need to call _exit rather than exit
+ */
+#ifdef STANDALONE
+#  define EXIT _exit
+#else
+#  define EXIT exit
+#endif /* STANDALONE */
+
+
             /* ^A */
 #define MMDF_SEP_CHAR   '\001'
 
@@ -211,6 +271,7 @@ extern int               pop_timeout;
 
 extern volatile int      hangup;
 
+
 #define pop_command         pop_parm[0]     /*  POP command is first token */
 #define pop_subcommand      pop_parm[1]     /*  POP XTND subcommand is the 
                                                 second token */
@@ -230,40 +291,56 @@ typedef enum {                              /*  POP processing states */
 } state;
 
 
-typedef enum {                              /* Authentication Method In Use  */
-    noauth,                                 /* not established               */
+typedef enum {                              /* Authentication Method In Use: */
+    noauth,                                 /*     not established           */
 #ifdef APOP
-    apop,                                   /* APOP                          */
+    apop,                                   /*     APOP                      */
 #endif
 #ifdef KERBEROS
-    kerberos,                               /* Kerberos [user/pass or        
-                                                         user/rpop]          */
+    kerberos,                               /*     Kerberos [user/pass or    
+                                                             user/rpop]      */
 #endif
 #ifdef SCRAM
-    scram_md5,                              /* SCRAM-MD5                     */
+    scram_md5,                              /*     SCRAM-MD5                 */
 #endif
-    plain                                   /* plain text [user/pass or      
-                                                           user/rpop]        */
+    plain                                   /*     plain text [user/pass or  
+                                                               user/rpop]    */
 } auth_type;
 
 
 typedef enum {                               /* Authentication Success  Substates */
-    auth=0,                                  /* AUTH  begun                       */
+    auth=0,                                  /*  AUTH  begun                      */
 #ifdef SCRAM
     clchg,                                   /*  Client challenge received        */
     clrsp,                                   /*  Client proof success             */
     clok,                                    /*  Client likes the server's stuff  */
 #endif
 #ifdef APOP
-    apopcmd,                                 /* APOP success under apop           */
+    apopcmd,                                 /*  APOP success under apop          */
 #endif
-    user,                                    /* USER under plain or kerberos      */
-    pass,                                    /* PASS under plain or kerberos      */
-    rpop,                                    /* RPOP under plain or kerberos      */
-    none                                     /* not established or don't care     */
+    user,                                    /*  USER under plain or kerberos     */
+    pass,                                    /*  PASS under plain or kerberos     */
+    rpop,                                    /*  RPOP under plain or kerberos     */
+    none                                     /*  not established or don't care    */
 } auth_state;
 
 
+typedef enum {                               /* Clear text passwords are:         */
+    ClearTextDefault = 0,                    /* - OK if nothing better available  */
+    ClearTextNever,                          /* - never OK                        */
+    ClearTextAlways,                         /* - OK even if APOP password exists */
+    ClearTextLocal                           /* - OK on 127.* (local loopback)    */
+} clear_text_type;
+
+
+typedef enum {                               /* Config file option is:         */
+    CfgBool = 1,                             /* - boolean                      */
+    CfgInt,                                  /* - integer                      */
+    CfgStr,                                  /* - string                       */
+    CfgBad                                   /* - INVALID VALUE                */
+} config_opt_type;
+
+
 typedef struct {                                /*  State information for 
                                                     each POP command */
     state       ValidCurrentState;              /*  The operating state of 
@@ -291,11 +368,21 @@ typedef struct {                                /*  Table of extensions */
                                                     the subcommand */
 } xtnd_table;
 
+
+typedef struct {                                /*  Table of config file settings */
+    char             *name;                     /*  The option name */
+    config_opt_type   type;                     /*  The option type */
+    BOOL              restricted;               /*  TRUE if user can't set */
+    int               value;                    /*  Value for switch */
+} config_table;
+
+
 typedef struct {
     char *auth_mechanism;                       /* Name of mechanism */
     int (*function)();                          /* Function that handles */
 } auth_table;
 
+
 typedef struct {                                /*  Message information */
     int         number;                         /*  Message number relative to 
                                                     the beginning of list */
@@ -309,13 +396,13 @@ typedef struct {                                /*  Message information */
                                                     lines in the body */
     long        offset;                         /*  Offset from beginning of 
                                                     file */
-    int         del_flag;                       /*  Flag indicating if message 
+    BOOL        del_flag;                       /*  Flag indicating if message 
                                                     is marked for deletion */
-    int         hide_flag;                      /*  Flag indicating if message
+    BOOL        hide_flag;                      /*  Flag indicating if message
                                                     is hidden but not deleted */
-    int         retr_flag;                      /*  Flag indicating if message 
+    BOOL        retr_flag;                      /*  Flag indicating if message 
                                                     was retrieved */
-    int         orig_retr_state;                /*  What was the state at the
+    BOOL        orig_retr_state;                /*  What was the state at the
                                                     start of this session.
                                                     Used for RSET cmd. */
     char       *uidl_str;                       /*  Cache of the UIDL str for
@@ -331,13 +418,16 @@ typedef struct CallStack {
 } CALLSTACK;
 
 struct _pop {                                   /*  POP parameter block */
-    int                 debug;                  /*  Debugging requested */
-    int                 xmitting;               /*  =1 xtnd xmit started */
-    int                 stats;                  /*  Stats requested */
-    int                 dirty;                  /*  Any mailbox changes? */
-    int                 kerberos;               /*  Flag to enable kerberos
+    BOOL                debug;                  /*  Debugging requested */
+    BOOL                xmitting;               /*  =1 xtnd xmit started */
+    BOOL                stats;                  /*  Stats requested */
+    BOOL                dirty;                  /*  Any mailbox changes? */
+    BOOL                kerberos;               /*  Flag to enable kerberos
                                                     authentication */
-    int                 server_mode;            /*  Default at compile time */
+    char            *   kerberos_service;       /*  Kerberos service
+                                                    being used */
+    BOOL                server_mode;            /*  Default at compile time */
+    int                 check_lock_refresh;     /*  Check for lock refresh every x msgs */
     char            *   myname;                 /*  The name of this POP 
                                                     daemon program */
     char            *   myhost;                 /*  The name of our host 
@@ -348,10 +438,18 @@ struct _pop {                                   /*  POP parameter block */
                                                     client IP address */
     unsigned short      ipport;                 /*  Client port for privileged 
                                                     operations */
-    int                 downcase_user;          /*  =1 to downcase user name */
+    BOOL                downcase_user;          /*  TRUE to downcase user name */
     char                user[MAXUSERNAMELEN];   /*  Name of the POP user */
+    
+#if     defined(__bsdi__) && _BSDI_VERSION >= 199608
+    char            *   style;                  /*  style of auth used */
+    char            *   challenge;              /*  challenge, if any */
+    login_cap_t     *   class;                  /*  user's class info */
+#endif /* __bsdi__) && _BSDI_VERSION >= 199608 */
+
     char                authid[MAXUSERNAMELEN]; /*  Authentication ID (SASL 
                                                     only) */
+    clear_text_type     AllowClearText;         /*  Setting for clear text passwords */
     state               CurrentState;           /*  The current POP operational
                                                     state */
     auth_type           AuthType;               /*  authentication type used */
@@ -397,11 +495,13 @@ struct _pop {                                   /*  POP parameter block */
 #  ifdef GDBM
     GDBM_FILE           bull_db;
 #  else
-    DBM            *    bull_db;                /*  Central Bulletin Database*/
+    DBM            *    bull_db;                /*  Central bulletin database*/
 #  endif
+    BOOL                bulldb_nonfatal;        /*  OK to not open bulldb ? */
+    int                 bulldb_max_tries;       /*  Max tries to open bulldb */
 #endif
 
-    int                 mangle_mime;            /*  turn all MIME into plain
+    BOOL                mangle_mime;            /*  turn all MIME into plain
                                                     text */
     char            *   mmdf_separator;         /*  string between messages */
     char                md5str[BUFSIZ];         /*  String used with the shared
@@ -416,6 +516,18 @@ struct _pop {                                   /*  POP parameter block */
     SCRAM_MD5_VRFY      scram_verifiers;        /* SCRAM-MD5 verifiers & salt  */
     SCRAM_MD5_INTEGRITY scram_mac;              /* SCRAM-MD5 MAC controls      */
 #endif /* SCRAM */
+
+#ifdef DO_TIMING
+    time_t              login_time;             /* Elapsed time to login */
+    time_t              init_time;              /* Elapsed time to init drop info */
+    time_t              clean_time;             /* Elapsed time to clean up */
+#endif /* DO_TIMING */
+
+#ifdef    DRAC_AUTH
+    char            *   drac_host;              /* Host which handles drac */
+#endif /* DRAC_AUTH */
+
+    BOOL                user_opts;              /* process ~/.qpopper-options ? */
 };
 
 typedef enum { HANGUP,  /* SIGHUP */ 
@@ -426,22 +538,40 @@ typedef enum { HANGUP,  /* SIGHUP */
 
 #ifdef KERBEROS
 
-#  ifndef KERBEROS_SERVICE
-#    define KERBEROS_SERVICE   "rcmd"
-#  endif
+#  if defined(KRB4) && defined(KRB5)
+#    error you can only use one of KRB4, KRB5
+#  endif /* KRB4 && KRB5 */
 
-#  ifdef SOLARIS2
-#    include <kerberos/krb.h>
-#  else /* not SOLARIS2 */
-#    ifdef BSDI
-#      include <kerberosIV/des.h>
-#      include <kerberosIV/krb.h>
-#    else /* not BSDI */
-#      include <krb.h>
-#    endif /* BSDI or not */
-#  endif /* SOLARIS2 or not */
+#  if !defined(KRB4) && !defined(KRB5)
+#    error you must use one of KRB4, KRB5
+#  endif /* !KRB4 && !KRB5 */
+
+#  ifdef KRB4
+#    ifndef KERBEROS_SERVICE
+#      define KERBEROS_SERVICE   "rcmd"
+#    endif
+
+#    ifdef SOLARIS2
+#      include <kerberos/krb.h>
+#    else /* not SOLARIS2 */
+#      ifdef __bsdi__
+#        include <kerberosIV/des.h>
+#        include <kerberosIV/krb.h>
+#      else /* not __bsdi__ */
+#        include <krb.h>
+#      endif /* __bsdi__ or not */
+#    endif /* SOLARIS2 or not */
 
    extern AUTH_DAT kdata;
+
+#  endif /* KRB4 */
+
+#  ifdef KRB5
+#    ifndef KERBEROS_SERVICE
+#      define KERBEROS_SERVICE "pop"
+#    endif
+#  endif /* KRB5 */
+
 #endif /* KERBEROS */
 
 #if defined(AUTHFILE)
@@ -452,44 +582,62 @@ typedef enum { HANGUP,  /* SIGHUP */
    extern int  checknonauthfile();
 #endif
 
-int  pop_dele(POP *p);
-int  pop_last(POP *p);
-int  pop_list(POP *p);
-int  pop_pass(POP *p);
-int  pop_rpop(POP *p);
-int  pop_apop(POP *p);
-int  pop_auth(POP *p);
-int  pop_quit(POP *p);
-int  pop_rset(POP *p);
-int  pop_send(POP *p);
-int  pop_stat(POP *p);
-int  pop_updt(POP *p);
-int  pop_user(POP *p);
-int  pop_xtnd(POP *p);
-int  pop_xmit(POP *p);
-int  pop_xmit_recv(POP *p, char *buffer);
-int  pop_xmit_exec(POP *p);
-int  pop_xlst(POP *p);
-int  pop_uidl(POP *p);
-int  pop_euidl(POP *p);
-int  pop_epop(POP *p);
-int  pop_scram(POP *p);
-int  pop_capa(POP *p);
-int  encode64(char *instr, int instrlen, char *outstr, int *outstrlen);
+int  pop_apop      (POP *p);
+int  pop_auth      (POP *p);
+int  pop_bull      (POP *p, struct passwd *pwp);
+int  pop_capa      (POP *p);
+int  pop_dele      (POP *p);
+int  pop_dropcopy  (POP *p, struct passwd *pwp);
+int  pop_epop      (POP *p);
+int  pop_euidl     (POP *p);
+int  pop_exit      (POP *p, EXIT_REASON e);
+int  pop_init      (POP *p, int argcount, char **argmessage);
+int  pop_last      (POP *p);
+int  pop_list      (POP *p);
+int  pop_mdef      (POP *p);
+int  pop_parse     (POP *p, char *buf);
+int  pop_pass      (POP *p);
+int  pop_quit      (POP *p);
+int  pop_restore   (POP *p);
+int  pop_rpop      (POP *p);
+int  pop_rset      (POP *p);
+int  pop_scram     (POP *p);
+int  pop_send      (POP *p);
+int  pop_sendline  (POP *p, char *buffer);
+int  pop_stat      (POP *p);
+int  pop_uidl      (POP *p);
+int  pop_updt      (POP *p);
+int  pop_user      (POP *p);
+int  pop_xlst      (POP *p);
+int  pop_xmit      (POP *p);
+int  pop_xmit_clean(POP *p);
+int  pop_xmit_exec (POP *p);
+int  pop_xmit_recv (POP *p, char *buffer);
+int  pop_xtnd      (POP *p);
+
+pop_result pop_config (POP *p, char *config_file, BOOL bUser);
+
 int  decode64(char *instr, int instrlen, char *outstr, int *outstrlen);
-int  pop_exit(POP *p, EXIT_REASON e);
-int  pop_mdef(POP *p);
+int  encode64(char *instr, int instrlen, char *outstr, int *outstrlen);
 
-int pop_log(POP *p, log_level  stat, const char  *fn, size_t ln, const char *format, ... );                  /* parameters for format string */
-int pop_msg(POP *p, pop_result stat, const char  *fn, size_t ln, const char *format, ... );
+int pop_log(POP *p, log_level  loglev, const char  *fn, size_t ln, const char *format, ... );                  /* parameters for format string */
+int pop_msg(POP *p, pop_result status, const char  *fn, size_t ln, const char *format, ... );
 
+int  mkstemp       (char *path);
+void pop_lower     (char *buf);
+
+int  check_group(POP *p, struct passwd *pwd, const char *group_name);
 int  isfromline(char *cp);
+void downcase_uname (POP *p, char *q);
+void trim_domain    (POP *p, char *q);
+char *get_state_name ( state curState );
+
+int  qpopper ( int argc, char *argv[] );
+
 extern char *pwerrmsg;
 
 #define pop_auth_fail   pop_msg
-/*
-extern int  pop_auth_fail();
-*/
+
 
 int          Push        (CALLSTACK *, FP);
 FP           Pop         (CALLSTACK *);
@@ -503,12 +651,6 @@ char        *mfgets      (char *s, int size, FILE *stream);
 char *get_time();
 extern char *ZONE;
 
-char *s;
-int size;
-FILE *stream;
-
-
-
 struct topper {
   int    lines_to_output;
   void **out_state;
@@ -532,9 +674,10 @@ typedef struct {
 MsgInfoList *msg_ptr ( POP *p, int msg_num );
 
 
+#define HERE    __FILE__,        __LINE__
+#define WHENCE  const char * fn, size_t ln
+
 #ifdef DEBUG
-#  define HERE    __FILE__,        __LINE__
-#  define WHENCE  const char * fn, size_t ln
 
 #  define DEBUG_LOG0(a,b) do {                                      \
         if ( (a)->debug )                                           \
@@ -560,9 +703,28 @@ MsgInfoList *msg_ptr ( POP *p, int msg_num );
         if ( (a)->debug )                                           \
             pop_log ( a, POP_DEBUG, HERE, b, a1, a2, a3, a4, a5);   \
     } while ( 0 );
+#  define DEBUG_LOG6(a,b,a1,a2,a3,a4,a5,a6) do {                    \
+        if ( (a)->debug )                                           \
+            pop_log ( a, POP_DEBUG, HERE, b, a1, a2, a3, a4, a5,    \
+                      a6);                                          \
+    } while ( 0 );
+#  define DEBUG_LOG7(a,b,a1,a2,a3,a4,a5,a6,a7) do {                 \
+        if ( (a)->debug )                                           \
+            pop_log ( a, POP_DEBUG, HERE, b, a1, a2, a3, a4, a5,    \
+                      a6, a7);                                      \
+    } while ( 0 );
+#  define DEBUG_LOG8(a,b,a1,a2,a3,a4,a5,a6,a7,a8) do {              \
+        if ( (a)->debug )                                           \
+            pop_log ( a, POP_DEBUG, HERE, b, a1, a2, a3, a4, a5,    \
+                      a6, a7, a8);                                  \
+    } while ( 0 );
+#  define DEBUG_LOG9(a,b,a1,a2,a3,a4,a5,a6,a7,a8,a9) do {           \
+        if ( (a)->debug )                                           \
+            pop_log ( a, POP_DEBUG, HERE, b, a1, a2, a3, a4, a5,    \
+                      a6, a7, a8, a9);                              \
+    } while ( 0 );
+
 #else
-#  define HERE    NULL,            0
-#  define WHENCE  const char * fn, size_t ln
 
 #  define DEBUG_LOG0(a,b) 
 #  define DEBUG_LOG1(a,b,a1) 
@@ -570,6 +732,10 @@ MsgInfoList *msg_ptr ( POP *p, int msg_num );
 #  define DEBUG_LOG3(a,b,a1,a2,a3) 
 #  define DEBUG_LOG4(a,b,a1,a2,a3,a4) 
 #  define DEBUG_LOG5(a,b,a1,a2,a3,a4,a5) 
+#  define DEBUG_LOG6(a,b,a1,a2,a3,a4,a5,a6)
+#  define DEBUG_LOG7(a,b,a1,a2,a3,a4,a5,a6,a7)
+#  define DEBUG_LOG8(a,b,a1,a2,a3,a4,a5,a6,a7,a8)
+#  define DEBUG_LOG9(a,b,a1,a2,a3,a4,a5,a6,a7,a8,a9)
 #endif
 
 
