@@ -10,16 +10,23 @@
  */
 
 /*
- *  Current version of this POP implementation
+ *  Current version of Qpopper
  */
 
-#define VERS_NUM          "3.0.2"
+#define VERS_NUM          "3.1.2"
 
 #ifdef KERBEROS
-#  define VERS_SUF1       "-krb-IV"
-#else
+#  ifdef KRB4
+#    define VERS_SUF1     "-krb-IV"
+#  endif /* KRB4 */
+
+#  ifdef KRB5
+#    define VERS_SUF1     "-krb-V"
+#  endif /* KRB5 */
+
+#else /* not KERBEROS */
 #  define VERS_SUF1       ""
-#endif
+#endif /* KERBEROS */
 
 #ifdef _DEBUG
 #  define VERS_SUF2       "_DEBUG"
@@ -27,8 +34,4 @@
 #  define VERS_SUF2       ""
 #endif
 
-#ifdef SHY
-#  define VERSION         "?"      VERS_SUF1 VERS_SUF2
-#else
-#  define VERSION         VERS_NUM VERS_SUF1 VERS_SUF2      
-#endif
+#define VERSION         VERS_NUM VERS_SUF1 VERS_SUF2      
